vec4 _baseColor;
vec3 _ambientColor;
vec3 _diffuseColor;

#if defined(SPECULAR)

vec3 _specularColor;

vec3 computeLighting(vec3 normalVector, vec3 lightDirection, float attenuation, vec3 cameraDirection)
{
    // Ambient
    _ambientColor = _baseColor.rgb * u_ambientColor;

    // Diffuse
#if defined(NO_DIFFUSE_SHADOWS)
    // todo: temporary workaround for u_lightDirection not propperly set in SDK
    lightDirection = normalize(vec3(0.2, 0.2, 2.0));
#endif
    float ddot = dot(normalVector, lightDirection);
    float diffuseIntensity = attenuation * ddot;
#if defined(NO_DIFFUSE_SHADOWS)
    diffuseIntensity = abs(diffuseIntensity);
#else
    diffuseIntensity = max(0.0, diffuseIntensity);
#endif
    _diffuseColor = u_lightColor * _baseColor.rgb * diffuseIntensity;

    // Specular
    vec3 halfVector = normalize(lightDirection + cameraDirection);
    float specularIntensity = attenuation * max(0.0, pow(abs(dot(normalVector, halfVector)), u_specularExponent));
    specularIntensity = max(0.0, specularIntensity);
    _specularColor = u_lightColor * u_specularColor.rgb * specularIntensity;
	
	return _ambientColor + _diffuseColor + _specularColor;
}

#else

vec3 computeLighting(vec3 normalVector, vec3 lightDirection, float attenuation)
{
    // Ambient
    _ambientColor = _baseColor.rgb * u_ambientColor;

    // Diffuse
#if defined(NO_DIFFUSE_SHADOWS)
    // todo: temporary workaround for u_lightDirection not propperly set in SDK
    lightDirection = normalize(vec3(0.2, 0.2, 2.0));
#endif
	float ddot = dot(normalVector, lightDirection);
    float diffuseIntensity = attenuation * ddot;
#if defined(NO_DIFFUSE_SHADOWS)
    diffuseIntensity = abs(diffuseIntensity);
#else
    diffuseIntensity = max(0.0, diffuseIntensity);
#endif
    _diffuseColor = u_lightColor * _baseColor.rgb * diffuseIntensity;
	
	return _ambientColor + _diffuseColor;
}

#endif

